// KinectGig.java
// Gesture tool for a gig (stage presentation)

import ch.aplu.util.*;
import ch.aplu.jaw.*;
import java.util.*;
import ch.aplu.kinect.*;
import java.awt.*;
import java.net.URL;
import javax.swing.JOptionPane;

public class KinectGig
{
  private final String version = "1.32";

  enum State
  {
    MOVING_RIGHT, MOVING_LEFT, // Moving right or left hand
    WAIT_FOR_START, // Wait for right hand raised
    WAIT_FOR_COMMAND, // Wait for next movement
    OUT_OF_RANGE // Person of of Kinect's range
  };
  //

  private final int speedLimit;
  private final int rightCommand;
  private final int leftCommand;
  String rightKeyCodeStr;
  String leftKeyCodeStr;
  private String dllPath =
    Kinect.is64bit() ? "KinectHandler64" : "KinectHandler";
  private int width = 640;
  private int height = 480;
  private Point3D[] joints = new Point3D[20];
  private ModelessOptionPane mop;
  private Robot robot;
  private State status = State.OUT_OF_RANGE;
  private final SoundPlayer rightClip;
  private final SoundPlayer leftClip;
  private final SoundPlayer failClip;
  private final SoundPlayer runClip;
  private final SoundPlayer dingClip;
  private final SoundPlayer infoClip;
  private final SoundPlayer readyClip;
  private final SoundPlayer stopClip;
  private static String iconResourcePath = "gifs/kinect.gif";

  public KinectGig()
  {
    KinectGigProperties props = new KinectGigProperties();
    speedLimit = props.getIntValue("SpeedLimit");
//    System.out.println("speedLimit: " + speedLimit);   
    rightKeyCodeStr = props.getStringValue("RightKeyCode");
    rightCommand = props.getKeyCode(rightKeyCodeStr);
//    System.out.println("rightCommand: " + rightCommand);   
    leftKeyCodeStr = props.getStringValue("LeftKeyCode");
    leftCommand = props.getKeyCode(leftKeyCodeStr);
//    System.out.println("leftCommand: " + leftCommand);   

    ClassLoader loader = getClass().getClassLoader();
    URL iconUrl = loader.getResource(iconResourcePath);
    readyClip = new SoundPlayer(loader.getResource("wav/ready.wav"));
    rightClip = new SoundPlayer(loader.getResource("wav/right.wav"));
    leftClip = new SoundPlayer(loader.getResource("wav/left.wav"));
    failClip = new SoundPlayer(loader.getResource("wav/fail.wav"));
    runClip = new SoundPlayer(loader.getResource("wav/run.wav"));
    dingClip = new SoundPlayer(loader.getResource("wav/ding.wav"));
    infoClip = new SoundPlayer(loader.getResource("wav/info.wav"));
    stopClip = new SoundPlayer(loader.getResource("wav/stop.wav"));
    mop = new ModelessOptionPane(10, 10, ""
      + "Initializing...", iconUrl);
    mop.setTitle("KinectGig V" + version + " (www.aplu.ch)");

    Kinect kinect =
      new Kinect(dllPath, "Video Frame", 400, 10, width / 4, height / 4,
      NativeHandler.WS_BORDER | NativeHandler.WS_VISIBLE);
    kinect.setWindowScaleFactor(4);

    if (!kinect.isInitialized())
    {
      Kinect.delay(1000);
      mop.setText("Kinect not present"
        + "\n- Not connected?"
        + "\n- Not powered?"
        + "\n- Already running?", false);
      infoClip.play();
      return;
    }
    else
      dingClip.play();
    for (int i = 0; i < 20; i++)
      joints[i] = new Point3D();

    try
    {
      robot = new Robot();
    }
    catch (AWTException ex)
    {
    }

    showStatus("No person in range     ", true);
    long oldTime = 0;
    double xRightOld = 0;
    double xLeftOld = 0;
    boolean isFirst = true;
    while (true)
    {
      int skeletonId = kinect.getJoints(joints, 20);  // Blocks max 200 ms
      if (skeletonId > -1)  // Valid
      {
        if (status == State.OUT_OF_RANGE)
        {
          switchState(State.WAIT_FOR_START);
          showStatus("Wait right hand up");
          readyClip.play();
        }
        long time = new Date().getTime();
        if (time - oldTime > 200)
        {
          oldTime = time;
          int rightHandIndex = SkeletonJoint.HAND_RIGHT.ordinal();
          int leftHandIndex = SkeletonJoint.HAND_LEFT.ordinal();
          double xRight = joints[rightHandIndex].x;
          double yRight = joints[rightHandIndex].y;
          double xLeft = joints[leftHandIndex].x;
          double yLeft = joints[leftHandIndex].y;
          if (status == State.WAIT_FOR_COMMAND)
          {
            if (yLeft < 100)  // Stop
            {
              status = State.WAIT_FOR_START;
              stopClip.play();
              showStatus("Wait right hand up");
            }
          }
          if (status == State.WAIT_FOR_START)
          {
            if (yRight < 100)  // Run
            {
              status = State.WAIT_FOR_COMMAND;
              runClip.play();
              showStatus("Wait for gesture");
              isFirst = true;
            }
          }
          else
          {
            double speedRight = (xRight - xRightOld) / 0.1;
            double speedLeft = (xLeft - xLeftOld) / 0.1;
            xRightOld = xRight;
            xLeftOld = xLeft;
            if (isFirst)  // First speed not valid (must init xOld)
              isFirst = false;
            else
            {
              if (speedRight > speedLimit)
                switchState(State.MOVING_RIGHT);
              else if (speedLeft < -speedLimit)
                switchState(State.MOVING_LEFT);
              else
                switchState(State.WAIT_FOR_COMMAND);
            }
          }
        }
      }
      else
        switchState(State.OUT_OF_RANGE);
    }
  }

  private void switchState(State s)
  {
    switch (s)
    {
      case WAIT_FOR_COMMAND:
        if (s != status)
          status = State.WAIT_FOR_COMMAND;
        break;
      case MOVING_RIGHT:
        if (s != status)
        {
          rightClip.play();
          showStatus("Last gesture: RIGHT");
          robot.keyPress(rightCommand);
          status = State.MOVING_RIGHT;
        }
        break;
      case MOVING_LEFT:
        if (s != status)
        {
          leftClip.play();
          showStatus("Last gesture: LEFT");
          robot.keyPress(leftCommand);
          status = State.MOVING_LEFT;
        }
        break;
      case OUT_OF_RANGE:
        if (s != status)
        {
          failClip.play();
          showStatus("No person in range");
          status = State.OUT_OF_RANGE;
        }
        break;
      case WAIT_FOR_START:
        if (s != status)
        {
          showStatus("Wait right hand up");
          status = State.WAIT_FOR_START;
        }
        break;
    }
  }

  protected static void showFatalError(String msg)
  {
    JOptionPane.showMessageDialog(null, msg);
    System.exit(0);
  }

  private void showStatus(String msg)
  {
    showStatus(msg, false);
  }

  private void showStatus(String msg, boolean adapt)
  {
    mop.setText("Speed Limit: " + speedLimit
      + "\nRight key code: " + rightKeyCodeStr
      + "\nLeft key code: " + leftKeyCodeStr
      + "\nCurrent status: " + msg, adapt);
  }

  public static void main(String args[])
  {
    new KinectGig();
  }

}
